from collections import OrderedDict

from django.conf.urls import url
from rest_framework.routers import (
    SimpleRouter as _SimpleRouter,
    APIRootView,
    SchemaView,
    SchemaGenerator
)
from rest_framework.settings import api_settings
from rest_framework.urlpatterns import format_suffix_patterns
from orm88.connector import ORM88


class SimpleRouter(_SimpleRouter):

    def get_default_basename(self, viewset):
        """
        If `basename` is not specified, attempt to automatically determine
        it from the viewset.
        """
        queryset = getattr(viewset, 'queryset', None)  # type: ORM88

        assert queryset is not None, (
            '`basename` argument not specified, '
            'and could not automatically determine '
            'the name from the viewset, as it does '
            'not have a `.queryset` attribute.'
        )

        return queryset._model_name.lower()


class DefaultRouter(SimpleRouter):
    """
        The default router extends the SimpleRouter, but also adds in a default
        API root view, and adds format suffix patterns to the URLs.
        """
    include_root_view = True
    include_format_suffixes = True
    root_view_name = 'api-root'
    default_schema_renderers = None
    APIRootView = APIRootView
    APISchemaView = SchemaView
    SchemaGenerator = SchemaGenerator

    def __init__(self, *args, **kwargs):
        if 'root_renderers' in kwargs:
            self.root_renderers = kwargs.pop('root_renderers')
        else:
            self.root_renderers = list(api_settings.DEFAULT_RENDERER_CLASSES)
        super().__init__(*args, **kwargs)

    def get_api_root_view(self, api_urls=None):
        """
        Return a basic root view.
        """
        api_root_dict = OrderedDict()
        list_name = self.routes[0].name
        for prefix, viewset, basename in self.registry:
            api_root_dict[prefix] = list_name.format(basename=basename)

        return self.APIRootView.as_view(api_root_dict=api_root_dict)

    def get_urls(self):
        """
        Generate the list of URL patterns, including a default root view
        for the API, and appending `.json` style format suffixes.
        """
        urls = super().get_urls()

        if self.include_root_view:
            view = self.get_api_root_view(api_urls=urls)
            root_url = url(r'^$', view, name=self.root_view_name)
            urls.append(root_url)

        if self.include_format_suffixes:
            urls = format_suffix_patterns(urls)

        return urls



from collections import OrderedDict
from typing import Dict

from rest_framework import serializers

from orm88.connector import ORMServices


class ModelSerializer(serializers.Serializer):
    map_fields = {
        'AutoField': serializers.IntegerField,
        'CharField': serializers.CharField,
        'IntegerField': serializers.IntegerField,
        'ListField': serializers.ListField,
        'OneToOneField': serializers.IntegerField,
        'ForeignKey': serializers.IntegerField
    }

    def get_fields(self) -> Dict:
        assert hasattr(self, 'Meta'), (
            'Class {serializer_class} missing "Meta" attribute'.format(
                serializer_class=self.__class__.__name__
            )
        )
        assert hasattr(self.Meta, 'model'), (
            'Class {serializer_class} missing "Meta.model" attribute'.format(
                serializer_class=self.__class__.__name__
            )
        )
        assert hasattr(self.Meta, 'fields'), (
            'Class {serializer_class} missing "Meta.fields" attribute'.format(
                serializer_class=self.__class__.__name__
            )
        )
        assert len(self.Meta.fields) > 0, (
            'Are you stupid, you\'re put a blank list on "Meta.fields"'
        )
        ret = super(ModelSerializer, self).get_fields()
        uncreated_fields = set(self.Meta.fields) - set(ret.keys())
        model_info = self.Meta.model.model_info
        model_fields = model_info.get('fields')
        model_related = model_info.get('related_names')
        if uncreated_fields:
            for uncreated_field in uncreated_fields:
                if uncreated_field in model_fields:
                    field = model_fields.get(uncreated_field).get('type')
                elif uncreated_field in model_related:
                    field = 'ListField'
                    if model_related.get(uncreated_field).get('type') == 'OneToOneRel':
                        field = 'IntegerField'
                else:
                    field = 'CharField'
                field_class = self.map_fields.get(field, serializers.CharField)
                ret.update({
                    uncreated_field: field_class()
                })
        for key, value in ret.items():
            value.bind(key, self)
        return ret

    def to_representation(self, instance):
        ret = OrderedDict()
        model_info = self.Meta.model.model_info
        model_fields = model_info.get('fields')
        for field, field_class in self.get_fields().items():
            attr = field_class.source or field
            try:
                if model_fields.get(attr).get('type') in ['ForeignKey', 'OneToOneField']:
                    attr = f"{attr}_id"
            except AttributeError:
                pass
            value = getattr(instance, attr, None)
            if isinstance(value, ORMServices):
                if attr in model_info.get('related_names'):
                    value = instance.reverse_related(attr)
            if field_class.__class__.__name__ == 'SerializerMethodField':
                value = instance
            if value is not None:
                value = field_class.to_representation(value)

            ret[field] = value
        return ret
